; 3-wire interface emulator
; for ATtiny2313
; written by Wan-Derer, oct-2013
; 
; Ports assignment:
; PB7 - STB, word srobe
; PB6 - SCK, bit strobe
; PB5...PB1 - SDT, data: PB1 outputs "1", PB2 outputs "2" etc
;
; PD6...PD3,PD1 - test out: 8 us period pulses (4 us "1", 4 us "0")
; PD2 - Master Clock return (for calibration)
; 
; fuse bytes: EXT=$FF, HIGH=$DF, LOW=$A4
;


	.include "tn2313Adef.inc"

; === Defines
	.def temp=R16
	.def FFmask=R17		; mask for Inverse port D outputs

	.def DataEndL=R18	; Constant = Data+DSize
	.def DataEndH=R19	;

	.def CycEndL=R20	; Constant = Data+CycleSize
	.def CycEndH=R21	; 

	.equ MCLK=8*1000000	; Master Clock is 8 MHz

	.equ Data=Correct*2	; Will output correct data
	.equ DSize=24		; Data Table Size
	.equ CycleSize=8500	; Cycle Size in 4us quantums (34ms=8500*4us)
	  

; === Code
	.cseg
; --- Interrupts Table
	.org 0
		rjmp reset
	.org INT0addr		; External Interrupt0
		reti
	.org INT1addr		; External Interrupt1
		reti
	.org ICP1addr		; Timer/Counter1 Capture
		reti
	.org OC1Aaddr		; Timer/Counter1 Compare Match A     
		reti
	.org OVF1addr		; Timer/Counter1 Overflow 
		reti
	.org OVF0addr		; Timer/Counter0 Overflow
		reti
	.org URXCaddr		; USART RX Complete Interrupt
		reti
	.org UDREaddr		; USART Data Register Empty Interrupt
		reti
	.org UTXCaddr		; USART TX Complete Interrupt 
		reti
	.org ACIaddr		; Analog Comparator Interrupt 
		reti
	.org PCIBaddr		; Pin Change Interrupt Request B
		reti
	.org OC1Baddr		; Timer/Counter1 Compare Match B
		reti
	.org OC0Aaddr		; Timer/Counter0 Compare Match A
		rjmp Handle4us
	.org OC0Baddr		; Timer/Counter0 Compare Match B
		reti
	.org USI_STARTaddr	; USI start interrupt
		reti
	.org USI_OVFaddr	; USI overflow interrupt
		reti
	.org ERDYaddr		; EEPROM write complete
		reti
	.org WDTaddr		; Watchdog Timer Interrupt
		reti
	.org PCIAaddr		; Pin Change Interrupt Request A
		reti
	.org PCIDaddr		; Pin Change Interrupt Request D
		reti

reset:		cli
; --- Stack Init
		ldi temp,RAMEND
		out SPL,temp

; --- Periferal Init
		ldi temp,$45	; Internal CLK calibration
		out OSCCAL,temp	;

	; --- Analog Comparator power off
		ldi temp,1<<ACD
		out ACSR,temp

	; --- Ports Init
		ser temp	;
		out DDRB,temp	; Configure port B as output
		out DDRD,temp	; Configure port D as output
		clr temp	;
		out PORTB,temp	; set outputs B to "0"
		out PORTD,temp	; set outputs D to "0"

	; --- Timer 0 Init
		ldi temp,1<<WGM01|0<<WGM00	; CTC mode
		out TCCR0A,temp			;
		ldi temp,1<<CS00		; Timer clock = Master clock
		out TCCR0B,temp			;
		ldi temp,4*MCLK/1000000-1	; Interrupts period = 4 us
		out OCR0A,temp			;
		ldi temp,1<<OCIE0A		; Interrupts enable by Timer 0 Compare Match A
		out TIMSK,temp			;

; --- Regs/Mem Init
	; --- mask for Inverse port D outputs
		ldi FFmask,$FF	
	; --- Pointer
		ldi ZL,low(Data-1)
		ldi ZH,high(Data-1)
	; --- Data+DSize
		ldi DataEndL,low(Data+DSize)
		ldi DataEndH,high(Data+DSize)
	; --- Data+CycleSize
		ldi CycEndL,low(Data+CycleSize)
		ldi CycEndH,high(Data+CycleSize)


; --- Start Program
		sei	; Interrupts Global Enable

loop:		rjmp loop

; --- Interrupt Handle
Handle4us:
		out PIND,FFmask	; Inverse port D outputs

		adiw ZL,1	; Inc Pointer

		cp ZL,DataEndL	; If Pointer <= DataEnd, Go to Data Output
		cpc ZH,DataEndH	;
		brcs DataOut	;

		cp ZL,CycEndL		; If Pointer <= DataEnd, DO a wait cycle (reti)
		cpc ZH,CycEndH		;
		brcs WaitCycle  	;
		
		ldi ZL,low(Data-1)	; If Pointer > DataEnd, DO a new cycle
		ldi ZH,high(Data-1)	;

WaitCycle:	reti

DataOut:	lpm temp,Z
		out PORTB,temp
		reti


Correct:	.db 0b01000000,0b00000000
		.db 0b00110000,0b01110000
		.db 0b00000000,0b00001100
		.db 0b01001100,0b00000000
		.db 0b00101010,0b01101010
		.db 0b00000000,0b01000000
		.db 0b00000000,0b00110000
		.db 0b01110000,0b00000000
		.db 0b00001100,0b01001100
		.db 0b00000000,0b00101010 
		.db 0b01101010,0b00000000
		.db 0b10000000,0b00000000 





Wrong:





